/*
  Crystal Design Component demo application 1:

  Copyright (c) 1997, SupraSoft Ltd

  Purpose of this program is to demonstrate
  - Basic report activation and de-activation (Button1Click)
  - Page navigation (SpeedButton1/SpeedButton2Click)
  - Event handling for error status (CrystalDesign1Error)
  - Event handling for "please wait" kind of feature with hourglass cursor
    (CrystalDesign1JobStarted, CrystalDesign1JobCompleted)
  - "Current page / Last Page" label showing (CrystalDesign1JobCompleted)
  - Zoom in/out report (ScrollBar1Change)
  - "Print setup" feature (Button2Click)
  - "Print a report" feature (Button3Click)

*/
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "demo1.h"

#include "CRYSPRNT.HPP"
#pragma link "CrysPrnt"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "Crystdsg"
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormCreate(TObject *Sender)
{
  AnsiString exePath;
  exePath = ExtractFilePath(Application->ExeName);

  // Report file AND database file are in the same place
  // with the exectuable file.
  CrystalDesign1->ReportLocation = exePath;
  CrystalDesign1->CommonTableLocations = exePath;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button1Click(TObject *Sender)
{
  // Activate or Deactivate the report (NOT operator)
  CrystalDesign1->Active = !CrystalDesign1->Active;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button2Click(TObject *Sender)
{
  // Show a print setup dialogbox using a method in CRYSPRNT module.
  // CRYSPRNT.HPP should be #included and #pragma linked to this program.
  // See the top of this file.

  // Please note that these changes will take effect when you next
  // time activate (or refresh) the report or print it to a printer.
  ShowReportPrintSetupDialog(CrystalDesign1);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Button3Click(TObject *Sender)
{
  // Show a standard print dialogbox using a method in CRYSPRNT module.
  ShowReportPrintDialog(CrystalDesign1);
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SpeedButton1Click(TObject *Sender)
{
  // Goto the previous page
  CrystalDesign1->ShowPrevPage();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SpeedButton2Click(TObject *Sender)
{
  // Goto the next page
  CrystalDesign1->ShowNextPage();
}
//---------------------------------------------------------------------------


void __fastcall TForm1::ScrollBar1Change(TObject *Sender)
{
  // Change a zoom factor based on scrollbar position
  CrystalDesign1->ZoomFactor = ScrollBar1->Position;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CrystalDesign1AfterActive(TObject *Sender)
{
  // If report was de-activated then clear page numbering text.
  // This event is triggered everytime when the state of Active
  // property was changed (after event).
  if(CrystalDesign1->Active == FALSE) Panel1->Caption = "";
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CrystalDesign1Error(TObject *Sender)
{
  // Show the last error text in a caption
  Label2->Caption = CrystalDesign1->LastErrorText;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CrystalDesign1JobStarted(TObject *Sender)
{
  // When a job is started, show a hourglass cursor.
  Screen->Cursor = crHourGlass;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::CrystalDesign1JobCompleted(TObject *Sender)
{
  // When job is completed, it's time to show a normal cursor
  Screen->Cursor = crDefault;

  // This event is called when a report job is completed.
  // The job can be page navigation, report activation, printing,
  // drill down operaton etc.. This is a good place to update,
  // for example, page numbering labels or clear "please wait" status.

  // Show a page numbering in the panel. If the report is not yet fully
  // processed (ie. total number of pages is inaccurate) then add
  // '+' char to end of the page numbering string.
  Panel1->Caption = IntToStr(CrystalDesign1->ReportDisplayPage) + " / " +
                     IntToStr(CrystalDesign1->ReportLastPage);

  if(CrystalDesign1->ReportPrintEnded == FALSE)
     Panel1->Caption = Panel1->Caption + "+";
}
//---------------------------------------------------------------------------



