{
  Demo Application 4: Memo and Image Database Fields

  PLEASE NOTE: This demo application uses Delphi/C++Builder
  example database (DBDEMOS) and its BIOLIFE table. This table
  has both memo and image database fields.

  PLEASE NOTE: 16-bit version of TCrystalDesign (Delphi1) doesn't
  support memo and blob fields.

  This application demonstrates how you can use TDataset
  objects even with memo and image (bitmap) fields.

  You will also learn how to use OnDatasetXXX events of
  TCrystalDesign component.

  This demo uses OnDatasetGetFieldData event handler to return
  a field value and current timestamp (Now function). You can
  utilize this feature to use efficient Delphi/C++Builder code
  to construct a field data.

  OnDatasetGetFieldDataBlob event handler is used to draw
  a bitmap and return it as a blob field value. You could utilize
  this feature to draw, for example, complex graphs and show
  them on a report.

  Copyright (c) 1999 Suprasoft Ltd. All rights reserved.
}
unit demo4;

interface

uses
  WinTypes, SysUtils, Classes, Graphics, Controls, Forms,
  DB, DBTables, ExtCtrls, StdCtrls,
  CrysPrnt,
  Crystdsg,      { TCrystalDesign }
  CrystDef;      { TCDCDatasetAttributes and other types }

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    CrystalDesign1: TCrystalDesign;
    Query1: TQuery;
    Database1: TDatabase;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    CheckBox1: TCheckBox;
    btnActivate: TButton;
    btnPrev: TButton;
    btnNext: TButton;
    ComboBox1: TComboBox;
    Label1: TLabel;
    procedure CrystalDesign1Error(Sender: TObject);
    procedure btnPrevClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnActivateClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ComboBox1Click(Sender: TObject);
    procedure CrystalDesign1DatasetGetFieldData(Sender, dataset,
      field: TObject; fieldIdx: Integer; var value: Variant;
      var processed: Boolean);
    procedure CrystalDesign1DatasetGetFieldDataBlob(Sender, dataset,
      field: TObject; fieldIdx: Integer; var value: TStream;
      var processed: Boolean);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
  { If database connection fails then please check }
  { that you have installed Delphi/C++Builder      }
  { demo databases and Biolife table.              }
  Database1.Connected := TRUE;
end;

procedure TForm1.CrystalDesign1Error(Sender: TObject);
begin
  Caption := CrystalDesign1.LastErrorText;
end;

procedure TForm1.ComboBox1Click(Sender: TObject);
begin
  CrystalDesign1.ZoomFactor := StrToIntDef(Combobox1.Text, 0);
end;

procedure TForm1.btnPrevClick(Sender: TObject);
begin
  CrystalDesign1.ShowPrevPage;
end;

procedure TForm1.btnNextClick(Sender: TObject);
begin
  CrystalDesign1.ShowNextPage;
end;

procedure TForm1.btnActivateClick(Sender: TObject);
var
  dsAttr: TCDCDatasetAttributes;
begin
  CrystalDesign1.Active := FALSE;

  { Component is not yet activated. Let's do it now. }
  Screen.Cursor := crHourglass;
 try
  Caption := 'Activating the component...';

  Query1.Open;

  { There is a bug in CR and its P2SMON.DLL file, which }
  { ignores the field names. It uses just field indexes.}

  { Note! 3rd (index 2) field on a RPT file is either   }
  { Memo or Blob field. TQuery doesn't have exactly     }
  { the same fields and in the same order than the      }
  { report expects to see them. We can use TField.Index }
  { property to change the order of fields in TQuery.   }

  { Note! OnDatasetGetField event could be used also    }
  { to "remap" fields without changing TField.Index.    }

  if RadioButton1.Checked then begin
     { Memo field report }
     CrystalDesign1.ReportName := 'memodb.rpt';
     Query1.FieldByName('Notes').Index := 2;
  end else begin
     { Image field report }
     CrystalDesign1.ReportName := 'imagedb.rpt';
     Query1.FieldByName('Image').Index := 2;
  end;


  { Both example reports works without bookmarks, }
  { so we can save a little time and ignore them. }
  { You can try and see if your reports work with }
  { dsAttrNoBookmarks attribute. Usually it does. }
  dsAttr := [dsAttrNoBookmarks];

  if CheckBox1.Checked then
     { If checkbox1 is checked then }
     { use OnDatasetXXX event handlers }
     Include(dsAttr, dsAttrUseEvents);

  CrystalDesign1.SetNthDataset(0, Query1);
  CrystalDesign1.SetNthDatasetAttributes(0, dsAttr);

  { Activate the component }
  CrystalDesign1.Active := TRUE;

 finally
  Screen.Cursor := crDefault;

  if CrystalDesign1.LastErrorCode = 0 then
     Caption := 'Successfully activated';
 end;
end;

procedure TForm1.CrystalDesign1DatasetGetFieldData(Sender, dataset,
  field: TObject; fieldIdx: Integer; var value: Variant;
  var processed: Boolean);
var
  text: string;
begin
  { If we have processed a certain field here then we have }
  { to set "processed" variable TRUE. This indicates that  }
  { the default behaviour should be skipped because we did }
  { what was necessary already here.
  { This applies also to all other OnDatasetXXX events     }
  { which have "processed" variable.                       }

  { If this is the second field and Random number 0..2       }
  { generates 0 then append current time stamp to the string }
  if (fieldIdx = 1) AND (Random(3) = 0) then begin
     text  := (field as TField).DisplayText;
     text  := text + ':  ' + DateTimeToStr(Now);

     value := text;
     processed := TRUE;
  end;
end;

procedure TForm1.CrystalDesign1DatasetGetFieldDataBlob(Sender, dataset,
  field: TObject; fieldIdx: Integer; var value: TStream;
  var processed: Boolean);
var
  bitmap: TBitmap;
begin
  { If this is image report and  Random number 0..2       }
  { generates 0 then draw a bitmap and return it as blob. }
  if (RadioButton2.Checked) AND (Random(2) = 0) then begin
     bitmap := TBitmap.Create;
    try
     bitmap.Width  := 150;
     bitmap.Height := 50 + Random(100);

     { Random color }
     bitmap.Canvas.Brush.Color := RGB(Random(256), Random(256), Random(256));

     if Random(2) = 0 then
        bitmap.Canvas.Rectangle(1, 1, bitmap.Width, bitmap.Height)
     else
        bitmap.Canvas.Ellipse(1, 1, bitmap.Width, bitmap.Height);

     bitmap.Canvas.Font.Style := [fsBold, fsItalic];
     bitmap.Canvas.TextOut(5, bitmap.Height div 2, (dataset as TDataset).Fields[1].DisplayText);

     { Blob is returned in "Value" stream. }
     { Let's ask bitmap to store itself into the stream }
     bitmap.SaveToStream(value);

     { Don't forget to set "processed := TRUE" or }
     { you won't see the bitmap on a report }
     processed := TRUE;

    finally
     bitmap.Free;
    end;
  end;
end;

end.
